var expect = require('chai').expect;
var util = require('../../lib/util.js');
var MakeDrive = require('../../../client/src');
var Filer = require('../../../lib/filer.js');

describe('MakeDrive Client - sync many dirs, many files', function(){
  var provider;

  beforeEach(function(done) {
    util.ready(function() {
      provider = new Filer.FileSystem.providers.Memory(util.username());
      done();
    });
  });
  afterEach(function() {
    provider = null;
  });

  function smallFile(number) {
    return '<!doctype html> '+
           '<head> '+
             '<meta charset="utf-8"> '+
             '<title> Small File ' + number + ' </title> '+
           '</head> '+
           '<body> '+
             '<p>This is small file ' + number + '.</p> '+
           '</body> '+
           '</html>';
  }

  /**
   * This test creates many dirs and files in each, syncs, and checks that
   * they exist on the server. It then removes them, and makes sure a
   * downstream sync brings them back.
   */
  it('should sync many dirs, many files', function(done) {
    util.authenticatedConnection(function( err, result ) {
      expect(err).not.to.exist;

      var fs = MakeDrive.fs({provider: provider, manual: true, forceCreate: true});
      var sync = fs.sync;

      // Make a layout with 25 dirs, each with sub-dirs, and files
      var layout = {};
      for(var i=0; i<25; i++) {
        for(var j=0; j<5; j++) {
          for(var k=0; k<3; k++) {
            layout['/' + i + '/' + j + '/' + k + '.html'] = smallFile(i);
          }
        }
      }

      sync.once('connected', function onConnected() {
        util.createFilesystemLayout(fs, layout, function(err) {
          expect(err).not.to.exist;

          sync.request();
        });
      });

      sync.once('completed', function onUpstreamCompleted() {
        util.ensureRemoteFilesystem(layout, result.jar, function(err) {
          expect(err).not.to.exist;
          sync.disconnect();
        });
      });

      sync.once('disconnected', function onDisconnected() {
        util.deleteFilesystemLayout(fs, null, function(err) {
          expect(err).not.to.exist;

          // Re-sync with server and make sure we get our deep dir back
          sync.once('connected', function onSecondDownstreamSync() {

            sync.once('disconnected', function onSecondDisconnected() {
              util.ensureFilesystem(fs, layout, function(err) {
                expect(err).not.to.exist;

                done();
              });
            });

            sync.disconnect();
          });

          // Get a new token for this second connection
          util.getWebsocketToken(result, function(err, result) {
            expect(err).not.to.exist;

            sync.connect(util.socketURL, result.token);
          });
        });
      });

      sync.connect(util.socketURL, result.token);
    });
  });

});
